// ----------------------------------------------------------------------------

// Access functions for the ORPSoC Verilator model: definition

// Copyright (C) 2008  Embecosm Limited <info@embecosm.com>

// Contributor Jeremy Bennett <jeremy.bennett@embecosm.com>

// This file is part of the cycle accurate model of the OpenRISC 1000 based
// system-on-chip, ORPSoC, built using Verilator.

// This program is free software: you can redistribute it and/or modify it
// under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or (at your
// option) any later version.

// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
// License for more details.

// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

// ----------------------------------------------------------------------------

// $Id: OrpsocAccess.h,v 1.1 2009/02/20 18:35:29 jeremybennett Exp $


#ifndef ORPSOC_ACCESS__H
#define ORPSOC_ACCESS__H

#include <stdint.h>

class Vorpsoc_fpga_top;
class Vorpsoc_fpga_top_orpsoc_fpga_top;
class Vorpsoc_fpga_top_or1200_ctrl;
class Vorpsoc_fpga_top_or1200_rfram_generic;


//! Access functions to the Verilator model

//! This class encapsulates access to the Verilator model, allowing other
//! Classes to access model state, without needing to be built within the
//! Verilator environment.
class OrpsocAccess
{
public:

  // Constructor
  OrpsocAccess (Vorpsoc_fpga_top *orpsoc_fpga_top);

  // Accessor functions
  bool      getWbFreeze ();
  uint32_t  getWbInsn ();
  uint32_t  getGpr (uint32_t regNum);

private:

  // Pointers to modules with accessor functions
  Vorpsoc_fpga_top_or1200_ctrl          *or1200_ctrl;
  Vorpsoc_fpga_top_or1200_rfram_generic *rf_a;

};	// OrpsocAccess ()

#endif	// ORPSOC_ACCESS__H
